@@ -132,6 +132,7 @@ tStrPos findHostStart(const mstring & sUri);
 #endif
 
 #define WITHLEN(x) x, (_countof(x)-1)
+#define MAKE_PTR_0_LEN(x) x, 0, (_countof(x)-1)
 
 //extern mstring sPathSep, sPathSepUnix, sCR, sCRLF;
 
@@ -159,24 +160,31 @@ tStrVec::size_type Tokenize(const mstring &in, LPCSTR sep, tStrVec & out, bool b
 bool ParseKeyValLine(const mstring & sIn, mstring & sOutKey, mstring & sOutVal);
 #define keyEq(a, b) (0 == strcasecmp((a), (b).c_str()))
 
+extern cmstring PROT_PFX_HTTPS, PROT_PFX_HTTP;
 
 class tHttpUrl
 {
+
+private:
+	mstring sPort;
+
 public:
 	bool SetHttpUrl(cmstring &uri, bool unescape = true);
 	mstring ToURI(bool bEscaped) const;
 	mstring sHost, sPath, sUserPass;
+
 #ifdef HAVE_SSL
 	bool bSSL=false;
-#endif
-	inline cmstring GetProtoPrefix() const
+	inline cmstring & GetProtoPrefix() const
 	{
-		return
-#ifdef HAVE_SSL
-			bSSL ? "https://" :
-#endif
-					"http://";
+		return bSSL ? PROT_PFX_HTTPS : PROT_PFX_HTTP;
 	}
+#else
+	inline cmstring & GetProtoPrefix() const
+	{
+		return PROT_PFX_HTTP;
+	}
+#endif
 
 	tHttpUrl & operator=(const tHttpUrl &a) 
 	{
@@ -215,8 +223,11 @@ public:
 #endif
 	}
 	inline cmstring& GetPort() const { return !sPort.empty() ? sPort : GetDefaultPortForProto(); }
-private:
-	mstring sPort;
+
+
+	inline tHttpUrl(cmstring &host, cmstring& port, bool ssl)
+	: sPort(port), sHost(host), bSSL(ssl) {};
+	inline tHttpUrl() =default;
 };
 
 #define POKE(x) for(;;) { ssize_t n=write(x, "", 1); if(n>0 || (EAGAIN!=errno && EINTR!=errno)) break;  }
@@ -298,6 +309,7 @@ mstring DosEscape(cmstring &s);
 
 // appears in the STL container?
 #define ContHas(stlcont, needle) ((stlcont).find(needle) != (stlcont).end())
+#define ContHasLinear(stlcont, needle) ((stlcont).end() != (std::find((stlcont).begin(), (stlcont).end(), needle)))
 
 #define StrHas(haystack, needle) (haystack.find(needle) != stmiss)
 #define StrHasFrom(haystack, needle, startpos) (haystack.find(needle, startpos) != stmiss)
@@ -448,13 +460,7 @@ static inline time_t GetTime()
 	return ::time(0);
 }
 
-// arbitrary chosen time declaring some date in far future
-// -1 is preserved for convenience reasons (avoid clashes with time(2) return code and similar uses)
-#ifndef PTHREAD_COND_TIMEDWAIT_TIME_RANGE_OK
-#define END_OF_TIME (time_t(MAX_VAL(int))-2)
-#else
-#define END_OF_TIME (MAX_VAL(time_t)-2)
-#endif
+static const time_t END_OF_TIME(MAX_VAL(time_t)-2);
 
 static inline uint FormatTime(char *buf, const time_t cur)
 {
