@@ -48,9 +48,8 @@ cmstring& GetFooter()
 	if(footer.empty())
 	{
 		footer = string("<hr><address>Server: ") + acfg::agentname +
-		" | <a href=\"https://flattr.com/thing/51105/Apt-Cacher-NG\">"
-			"Flattr it!</a> | <a href=\"http://www.unix-ag.uni-kl.de/~bloch/acng/\">"
-			"Apt-Cacher NG homepage</a></address>";
+		"&nbsp;&nbsp;|&nbsp;&nbsp;<a href=\"https://flattr.com/thing/51105/Apt-Cacher-NG\">Flattr this!"
+    "</a>&nbsp;&nbsp;|&nbsp;&nbsp;<a href=\"http://www.unix-ag.uni-kl.de/~bloch/acng/\">Apt-Cacher NG homepage</a></address>";
 	}
 	return footer;
 }
@@ -110,7 +109,7 @@ void tSpecOpDetachable::Run()
 		pTracked=g_pTracker.lock();
 		if(!pTracked) // ok, not running yet -> become the log source then
 		{
-			m_pTracker.reset(new tProgressTracker);
+			m_pTracker = make_shared<tProgressTracker>();
 			msg.clear();
 			time_t nMaintId=time(0);
 
