@@ -23,8 +23,6 @@ using namespace std;
 // evil hack to simulate random disconnects
 //#define DISCO_FAILURE
 
-typedef std::pair<const tHttpUrl*,bool> tHostIsproxy;
-
 static cmstring sGenericError("567 Unknown download error occured");
 
 std::atomic_uint g_nDlCons(0);
@@ -78,9 +76,9 @@ struct tDlJob
 	 * Returns a reference to http url where host and port and protocol match the current host
 	 * Other fields in that member have undefined contents. ;-)
 	 */
-	inline const tHttpUrl *GetPeerHost()
+	inline const tHttpUrl& GetPeerHost()
 	{
-		return m_pCurBackend ? m_pCurBackend : &m_remoteUri;
+		return m_pCurBackend ? *m_pCurBackend : m_remoteUri;
 	}
 
 	inline acfg::tRepoData::IHookHandler * GetConnStateTracker()
@@ -207,26 +205,6 @@ struct tDlJob
 		return true;
 	}
 
-	inline tHostIsproxy GetConnectHost()
-	{
-		if (!m_parent.m_bProxyTot)
-		{
-			// otherwise consider using proxy
-			if (m_pRepoDesc && m_pRepoDesc->m_pProxy)
-			{
-				// do what the specific entry says
-				if(m_pRepoDesc->m_pProxy->sHost.empty())
-					return make_pair(GetPeerHost(), false);
-				return make_pair(m_pRepoDesc->m_pProxy, true);
-			}
-			if (!acfg::proxy_info.sHost.empty())
-				return make_pair(& acfg::proxy_info, true);
-			// ok, no proxy...
-		}
-		return make_pair(GetPeerHost(), false);
-	}
-
-
 	bool SetupJobConfig(mstring& sReasonMsg, decltype(dlcon::m_blacklist) &blacklist)
 	{
 		LOGSTART("dlcon::SetupJobConfig");
@@ -263,7 +241,7 @@ struct tDlJob
 
 		// ok, not backend mode. Check the mirror data (vs. blacklist)
 		auto bliter = blacklist.find(
-				make_pair(GetPeerHost()->sHost, GetPeerHost()->GetPort()));
+				make_pair(GetPeerHost().sHost, GetPeerHost().GetPort()));
 		if (bliter == blacklist.end())
 			return true;
 
@@ -272,18 +250,14 @@ struct tDlJob
 	}
 
 	// needs connectedHost, blacklist, output buffer from the parent, proxy mode?
-	inline void AppendRequest(tSS &head, cmstring &xff)
+	inline void AppendRequest(tSS &head, cmstring &xff, const tHttpUrl *proxy)
 	{
 		LOGSTART("tDlJob::AppendRequest");
 
 		head << (m_pStorage->m_bHeadOnly ? "HEAD " : "GET ");
 
-		tHostIsproxy conHostInfo = GetConnectHost();
-
-		if (conHostInfo.second)
-		{
+		if (proxy)
 			head << RemoteUri(true);
-		}
 		else // only absolute path without scheme
 		{
 			if (m_pCurBackend) // base dir from backend definition
@@ -294,15 +268,15 @@ struct tDlJob
 
 		ldbg(RemoteUri(true));
 
-		head << " HTTP/1.1\r\n" << acfg::agentheader << "Host: " << GetPeerHost()->sHost << "\r\n";
+		head << " HTTP/1.1\r\n" << acfg::agentheader << "Host: " << GetPeerHost().sHost << "\r\n";
 
-		if (conHostInfo.second) // proxy stuff, and add authorization if there is any
+		if (proxy) // proxy stuff, and add authorization if there is any
 		{
 			ldbg("using proxy");
-			if(!conHostInfo.first->sUserPass.empty())
+			if(!proxy->sUserPass.empty())
 			{
 				head << "Proxy-Authorization: Basic "
-						<< EncodeBase64Auth(conHostInfo.first->sUserPass) << "\r\n";
+						<< EncodeBase64Auth(proxy->sUserPass) << "\r\n";
 			}
 			// Proxy-Connection is a non-sensical copy of Connection but some proxy
 			// might listen only to this one so better add it
@@ -310,7 +284,7 @@ struct tDlJob
 									: "Proxy-Connection: close\r\n");
 		}
 
-		const tHttpUrl& pSourceHost(* GetPeerHost());
+		const auto& pSourceHost = GetPeerHost();
 		if(!pSourceHost.sUserPass.empty())
 		{
 			head << "Authorization: Basic "
@@ -697,9 +671,8 @@ bool dlcon::AddJob(tFileItemPtr m_pItem, const tHttpUrl *pForcedUrl,
 	LOGSTART2("dlcon::EnqJob", todo->m_remoteUri.ToURI(false));
 */
 	m_qNewjobs.push_back(
-			tDlJobPtr(
-					new tDlJob(this, m_pItem, pForcedUrl, pBackends, sPatSuffix,
-							m_bManualMode ? ACFG_REDIRMAX_DEFAULT : acfg::redirmax)));
+			make_shared<tDlJob>(this, m_pItem, pForcedUrl, pBackends, sPatSuffix,
+							m_bManualMode ? ACFG_REDIRMAX_DEFAULT : acfg::redirmax));
 	wake();
 	return true;
 }
@@ -1075,9 +1048,25 @@ void dlcon::WorkLoop()
 	auto BlacklistMirror = [&](tDlJobPtr & job)
 	{
 		LOGSTART2("BlacklistMirror", "blacklisting " <<
-				job->GetPeerHost()->ToURI(false));
-		m_blacklist[std::make_pair(job->GetPeerHost()->sHost,
-				job->GetPeerHost()->GetPort())] = sErrorMsg;
+				job->GetPeerHost().ToURI(false));
+		m_blacklist[std::make_pair(job->GetPeerHost().sHost,
+				job->GetPeerHost().GetPort())] = sErrorMsg;
+	};
+
+	auto prefProxy = [&](tDlJobPtr& cjob) -> const tHttpUrl*
+	{
+		if(m_bProxyTot)
+			return nullptr;
+
+		if(cjob->m_pRepoDesc && cjob->m_pRepoDesc->m_pProxy
+				&& !cjob->m_pRepoDesc->m_pProxy->sHost.empty())
+		{
+			return cjob->m_pRepoDesc->m_pProxy;
+		}
+		if(!acfg::proxy_info.sHost.empty())
+			return &acfg::proxy_info;
+
+		return nullptr;
 	};
 
 	while(true) // outer loop: jobs, connection handling
@@ -1131,29 +1120,62 @@ void dlcon::WorkLoop()
         			goto go_select; // nothing left, might receive new jobs soon
         		}
 
-        		bool bUsed=false;
+				bool bUsed = false;
+				ASSERT(!m_qNewjobs.empty());
+
+				auto doconnect = [&](const tHttpUrl& tgt, int timeout, bool fresh)
+				{
+					return tcpconnect::CreateConnected(tgt.sHost,
+							tgt.GetPort(),
+							sErrorMsg,
+							&bUsed,
+							m_qNewjobs.front()->GetConnStateTracker(),
 #ifdef HAVE_SSL
-#define doconnect(x, t) tcpconnect::CreateConnected(x->sHost, x->GetPort(), \
-				sErrorMsg, &bUsed, m_qNewjobs.front()->GetConnStateTracker(), x->bSSL, t)
+						tgt.bSSL,
 #else
-#define doconnect(x, t) tcpconnect::CreateConnected(x->sHost, x->GetPort(), \
-				sErrorMsg, &bUsed, m_qNewjobs.front()->GetConnStateTracker(), false, t)
-
+						false
 #endif
-        		ASSERT(!m_qNewjobs.empty());
-        		tHostIsproxy conHost = m_qNewjobs.front()->GetConnectHost();
-        		con = doconnect( conHost.first, (conHost.second && acfg::optproxytimeout>0)
-        				? acfg::optproxytimeout : acfg::nettimeout);
+						timeout, fresh);
+			}	;
 
-        		if(!con && acfg::optproxytimeout>0)
+				auto& cjob = m_qNewjobs.front();
+				auto proxy = m_bProxyTot ? nullptr : prefProxy(cjob);
+				auto& peerHost = cjob->GetPeerHost();
+
+				if(peerHost.bSSL)
+				{
+					if(proxy)
+					{
+						con = doconnect(*proxy, acfg::optproxytimeout > 0 ?
+								acfg::optproxytimeout : acfg::nettimeout, false);
+						if(con)
+						{
+							if(!con->StartTunnel(peerHost, sErrorMsg, & proxy->sUserPass, true))
+								con.reset();
+						}
+					}
+					else
+						con = doconnect(peerHost, acfg::nettimeout, false);
+				}
+				else
+				{
+					if(proxy)
+					{
+						con = doconnect(*proxy, acfg::optproxytimeout > 0 ?
+								acfg::optproxytimeout : acfg::nettimeout, false);
+					}
+					else
+						con = doconnect(peerHost, acfg::nettimeout, false);
+				}
+
+        		if(!con && proxy && acfg::optproxytimeout>0)
         		{
         			ldbg("optional proxy broken, disable");
-        			m_bProxyTot=true;
-        			con = doconnect(m_qNewjobs.front()->GetPeerHost(), acfg::nettimeout);
+        			m_bProxyTot = true;
+        			proxy = nullptr;
+        			con = doconnect(peerHost, acfg::nettimeout, false);
         		}
 
-        		nLostConTolerance = MAX_RETRY + bUsed;
-
         		ldbg("connection valid? " << bool(con) << " was fresh? " << !bUsed);
 
         		if(con)
@@ -1170,7 +1192,7 @@ void dlcon::WorkLoop()
         		}
         		else
         		{
-        			BlacklistMirror(m_qNewjobs.front());
+        			BlacklistMirror(cjob);
         			continue; // try the next backend
         		}
         	}
@@ -1182,35 +1204,42 @@ void dlcon::WorkLoop()
         	{
    				tDlJobPtr &cjob = m_qNewjobs.front();
 
-        		bool bGoodConfig = cjob->SetupJobConfig(sErrorMsg, m_blacklist);
-
-        		/*
-        		ldbg("target: " << cjob->GetPeerName() << " vs " << con->GetHostname()
-        				<< ", ports: " << cjob->GetPeerPort() << " vs " << con->GetPort()
-        				<< ", good config: " << bGoodConfig);
-*/
-
-        		if(!bGoodConfig)
+        		if(!cjob->SetupJobConfig(sErrorMsg, m_blacklist))
         		{
         			// something weird happened to it, drop it and let the client care
         			m_qNewjobs.pop_front();
         			continue;
         		}
 
-				// needs to send them for the connected target host
-        		tHostIsproxy hostNew=cjob->GetConnectHost();
-        		if(hostNew.first->sHost != con->GetHostname() ||
-        				hostNew.first->GetPort() != con->GetPort())
+        		auto& tgt=cjob->GetPeerHost();
+        		// good case, direct or tunneled connection
+        		bool match=(tgt.sHost == con->GetHostname() && tgt.GetPort() == con->GetPort());
+        		const tHttpUrl * proxy = nullptr; // to be set ONLY if PROXY mode is used
+
+        		// if not exact and can be proxied, and is this the right proxy?
+        		if(!match)
         		{
-        			LOG("host mismatch," << hostNew.first->sHost << ":" <<
-        					hostNew.first->GetPort() <<
-        					" vs. " << con->GetHostname() << ":"<<con->GetPort() <<
-        					" -- stop sending requesting for now");
+        			proxy = prefProxy(cjob);
+        			if(proxy)
+        			{
+        				/*
+        				 * SSL over proxy uses HTTP tunnels (CONNECT scheme) so the check
+        				 * above should have matched before.
+        				 */
+        				if(!tgt.bSSL)
+        					match=(proxy->sHost == con->GetHostname() && proxy->GetPort() == con->GetPort());
+        			}
+        			// else... host changed and not going through the same proxy -> fail
+        		}
+
+        		if(!match)
+        		{
+        			LOG("host mismatch, new target: " << tgt.sHost << ":" << tgt.GetPort());
         			bStopRequesting=true;
         			break;
         		}
 
-				cjob->AppendRequest(m_sendBuf, m_sXForwardedFor);
+				cjob->AppendRequest(m_sendBuf, m_sXForwardedFor, proxy);
 				LOG("request added to buffer");
 				inpipe.push_back(cjob);
 				m_qNewjobs.pop_front();
@@ -1221,7 +1250,6 @@ void dlcon::WorkLoop()
 					--m_nTempPipelineDisable;
 					break;
 				}
-
         	}
         }
 
@@ -1262,7 +1290,6 @@ void dlcon::WorkLoop()
 			}
 		}
 
-
         /*
          * Here we go if the inpipe is still not processed or there have been errors
          * needing special handling.
@@ -1290,7 +1317,10 @@ void dlcon::WorkLoop()
 		{
 			// disconnected by OS... give it a chance, or maybe not...
 			if (--nLostConTolerance <= 0)
+			{
 				BlacklistMirror(inpipe.front());
+				nLostConTolerance=MAX_RETRY;
+			}
 
 			timespec sleeptime = { 0, 325000000 };
 			nanosleep(&sleeptime, NULL);
@@ -1305,7 +1335,6 @@ void dlcon::WorkLoop()
 
         if(loopRes & (HINT_DONE|HINT_MORE))
         {
-        	nLostConTolerance=MAX_RETRY;
         	sErrorMsg.clear();
         	continue;
         }
